@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -73,6 +73,7 @@ struct arithmeticparams
   struct gal_arithmetic_set_params setprm; /* Parameters for 'set-'.    */
 
   /* Input: */
+  char          *arguments;  /* File containing arguments to be used.   */
   gal_list_str_t     *hdus;  /* List of all given HDU strings.          */
   gal_list_str_t   *tokens;  /* List of all arithmetic tokens.          */
   char            *wcsfile;  /* File to use for output's WCS.           */
@@ -82,12 +83,19 @@ struct arithmeticparams
   char          *globalhdu;  /* Single HDU for all inputs.              */
   uint8_t      onedasimage;  /* Write 1D outputs as an image not table. */
   uint8_t     onedonstdout;  /* Write 1D outputs on stdout, not table.  */
+  gal_list_str_t *metaname;  /* FITS name (EXTNAME keyword) of output.  */
+  gal_list_str_t *metaunit;  /* FITS name (BUNIT keyword) of output.    */
+  gal_list_str_t *metacomment; /* FITS comment of output.               */
+  uint8_t         writeall;  /* Write all outputs.                      */
+  uint8_t           append;  /* Don't delete output file.               */
 
   /* Operating mode: */
   int        wcs_collapsed;  /* If the internal WCS is already collapsed.*/
 
   /* Internal: */
+  uint8_t          envseed;  /* To setup the random number generator.   */
   struct operand *operands;  /* The operands linked list.               */
+  int     outnamerequested;  /* ==1 if the user has given '--otuput'.   */
   time_t           rawtime;  /* Starting time of the program.           */
 };
 
