@@ -6,7 +6,7 @@ Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
      Pedram Ashofteh-Ardakani <pedramardakani@pm.me>
-Copyright (C) 2017-2022 Free Software Foundation, Inc.
+Copyright (C) 2017-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -103,7 +103,7 @@ fits_print_extension_info(struct fitsparams *p)
   /* Open the FITS file and read the first extension type, upon moving to
      the next extension, we will read its type, so for the first we will
      need to do it explicitly. */
-  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY, 1);
+  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY, 1, "NONE");
   if (fits_get_hdu_type(fptr, &hdutype, &status) )
     gal_fits_io_error(status, "reading first extension");
 
@@ -178,7 +178,7 @@ fits_print_extension_info(struct fitsparams *p)
         }
 
 
-      /* Read the extension name*/
+      /* Read the extension name. */
       fits_read_keyword(fptr, "EXTNAME", extname, NULL, &status);
       switch(status)
         {
@@ -206,7 +206,7 @@ fits_print_extension_info(struct fitsparams *p)
         case 0:
           /* For tables, we don't bother reading 'BUNIT', because it is
              meaningless even if it exists (each colum has its separate
-             units). So 'status' will be zero in this case.*/
+             units). So 'status' will be zero in this case. */
           if(hdutype==IMAGE_HDU)
             gal_fits_key_clean_str_value(bunit);
           else /* Its a table HDU. */
@@ -308,11 +308,12 @@ fits_print_extension_info(struct fitsparams *p)
         printf("           ('%s': no unit in HDU metadata, or "
                "HDU is a table)\n", GAL_BLANK_STRING);
       if(hascomments)
-        printf(" Column 6: Comments about the HDU (e.g., if its HEALpix, or "
-               "etc).\n");
+        printf(" Column 6: Comments about the HDU (e.g., if its HEALpix, "
+               "or etc).\n");
       printf("-----\n");
     }
-  gal_table_write(cols, NULL, NULL, GAL_TABLE_FORMAT_TXT, NULL, NULL, 0);
+  gal_table_write(cols, NULL, NULL, GAL_TABLE_FORMAT_TXT, NULL, NULL,
+                  0, 0);
   gal_list_data_free(cols);
 }
 
@@ -327,7 +328,7 @@ fits_hdu_number(struct fitsparams *p)
   int numhdu, status=0;
 
   /* Read the first extension (necessary for reading the rest). */
-  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY, 1);
+  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY, 1, "NONE");
 
   /* Get the number of HDUs. */
   if( fits_get_num_hdus(fptr, &numhdu, &status) )
@@ -347,7 +348,20 @@ fits_hdu_number(struct fitsparams *p)
 static void
 fits_datasum(struct fitsparams *p)
 {
-  printf("%ld\n", gal_fits_hdu_datasum(p->input->v, p->cp.hdu));
+  char *out;
+
+  if(p->datasum)
+    printf("%ld\n", gal_fits_hdu_datasum(p->input->v, p->cp.hdu, "--hdu"));
+  else if(p->datasumencoded)
+    {
+      out=gal_fits_hdu_datasum_encoded(p->input->v, p->cp.hdu, "--hdu");
+      printf("%s\n", out);
+      free(out);
+    }
+  else
+    error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' to fix "
+          "this bug! Both 'datasum' and 'datasumencoded' are zero",
+          __func__, PACKAGE_BUGREPORT);
 }
 
 
@@ -363,7 +377,7 @@ fits_read_check_wcs(struct fitsparams *p, size_t *ndim,
 
   /* Read the desired WCS. */
   wcs=gal_wcs_read(p->input->v, p->cp.hdu, p->cp.wcslinearmatrix,
-                   0, 0, &nwcs);
+                   0, 0, &nwcs, "--hdu");
 
   /* If a WCS doesn't exist, let the user know and return. */
   if(wcs)
@@ -393,7 +407,7 @@ fits_pixelscale(struct fitsparams *p)
 
   /* If not in quiet-mode, print some extra information. We don't want the
      last number to have a space after it, so we'll write the last one
-     outside the loop.*/
+     outside the loop. */
   if(p->cp.quiet==0)
     {
       printf("Basic information for --pixelscale (remove extra info "
@@ -513,7 +527,7 @@ fits_pixelarea(struct fitsparams *p)
   /* Calculate the pixel-scale in each dimension. */
   pixelscale=gal_wcs_pixel_scale(wcs);
 
-  /* Calcluate the pixel area (currently only in arcsec^N) */
+  /* Calcluate the pixel area (currently only in arcsec^N). */
   area=1;
   for(i=0;i<ndim;++i) area *= pixelscale[i]*3600;
   printf("%g\n", area);
@@ -535,7 +549,7 @@ fits_skycoverage(struct fitsparams *p)
 
   /* Find the coverage. */
   if( gal_wcs_coverage(p->input->v, p->cp.hdu, &ndim,
-                       &center, &width, &min, &max)==0 )
+                       &center, &width, &min, &max, "--hdu")==0 )
     error(EXIT_FAILURE, 0, "%s (hdu %s): is not usable for finding "
           "sky coverage (either doesn't have a WCS, or isn't an image "
           "or cube HDU with 2 or 3 dimensions", p->input->v, p->cp.hdu);
@@ -559,27 +573,27 @@ fits_skycoverage(struct fitsparams *p)
       switch(ndim)
         {
         case 2:
-          printf("  Center: %-15.10g%-15.10g\n", center[0], center[1]);
-          printf("  Width:  %-15.10g%-15.10g\n", width[0],  width[1]);
+          printf("  Center: %-15.10g %-15.10g\n", center[0], center[1]);
+          printf("  Width:  %-15.10g %-15.10g\n", width[0],  width[1]);
           break;
         case 3:
-          printf("  Center: %-15.10g%-15.10g%-15.10g\n", center[0],
+          printf("  Center: %-15.10g %-15.10g %-15.10g\n", center[0],
                  center[1], center[2]);
-          printf("  width:  %-15.10g%-15.10g%-15.10g\n", width[0],
+          printf("  width:  %-15.10g %-15.10g %-15.10g\n", width[0],
                  width[1], width[2]);
           break;
         default:
-          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
-                "the problem. 'ndim' value %zu is not recognized", __func__,
-                PACKAGE_BUGREPORT, ndim);
+          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
+                "fix the problem. 'ndim' value %zu is not recognized",
+                __func__, PACKAGE_BUGREPORT, ndim);
         }
 
       /* For the range type of coverage. */
       wcs=gal_wcs_read(p->input->v, p->cp.hdu, p->cp.wcslinearmatrix,
-                       0, 0, &nwcs);
+                       0, 0, &nwcs, "--hdu");
       printf("\nSky coverage by range along dimensions:\n");
       for(i=0;i<ndim;++i)
-        printf("  %-8s %-15.10g%-15.10g\n", gal_wcs_dimension_name(wcs, i),
+        printf("  %-8s %-15.10g %-15.10g\n", gal_wcs_dimension_name(wcs, i),
                min[i], max[i]);
       wcsfree(wcs);
     }
@@ -633,6 +647,10 @@ fits_certain_hdu(struct fitsparams *p, int list1has0,
   gal_list_str_t *names=NULL;
   int has=0, naxis, hducounter=1, hdutype, status=0;
 
+  /* Make sure the given file exists: CFITSIO adds '.gz' silently (see more
+     in the comments within 'gal_fits_hdu_open'). */
+  gal_checkset_check_file(p->input->v);
+
   /* Open the FITS file. */
   fits_open_file(&fptr, p->input->v, READONLY, &status);
   gal_fits_io_error(status, NULL);
@@ -698,6 +716,9 @@ fits_certain_hdu(struct fitsparams *p, int list1has0,
   status=0;
   fits_close_file(fptr, &status);
 
+  /* Reverse the list so they are in the same order as they were found. */
+  gal_list_str_reverse(&names);
+
   /* Print the result. */
   if( list1has0 )
     {
@@ -718,6 +739,10 @@ fits_list_all_hdus(struct fitsparams *p)
   gal_list_str_t *names=NULL;
   int hducounter=1, hdutype, status=0;
 
+  /* Make sure the given file exists: CFITSIO adds '.gz' silently (see more
+     in the comments within 'gal_fits_hdu_open'). */
+  gal_checkset_check_file(p->input->v);
+
   /* Open the FITS file. */
   fits_open_file(&fptr, p->input->v, READONLY, &status);
   gal_fits_io_error(status, NULL);
@@ -760,7 +785,7 @@ fits_hdu_remove(struct fitsparams *p, int *r)
       hdu=gal_list_str_pop(&p->remove);
 
       /* Open the FITS file at the specified HDU. */
-      fptr=gal_fits_hdu_open(p->input->v, hdu, READWRITE, 1);
+      fptr=gal_fits_hdu_open(p->input->v, hdu, READWRITE, 1, "--remove");
 
       /* Delete the extension. */
       if( fits_delete_hdu(fptr, &hdutype, &status) )
@@ -777,7 +802,7 @@ fits_hdu_remove(struct fitsparams *p, int *r)
 
 
 /* This is similar to the library's 'gal_fits_open_to_write', except that
-   it won't create an empty first extension.*/
+   it won't create an empty first extension. */
 fitsfile *
 fits_open_to_write_no_blank(char *filename)
 {
@@ -813,6 +838,7 @@ fits_hdu_copy(struct fitsparams *p, int cut1_copy0, int *r)
   char *hdu;
   int status=0, hdutype;
   fitsfile *in, *out=NULL;
+  char *hopt = cut1_copy0 ? "--cut" : "--copy";
   gal_list_str_t *list = cut1_copy0 ? p->cut : p->copy;
 
   /* Copy all the given extensions. */
@@ -823,11 +849,11 @@ fits_hdu_copy(struct fitsparams *p, int cut1_copy0, int *r)
 
       /* Open the FITS file at the specified HDU. */
       in=gal_fits_hdu_open(p->input->v, hdu,
-                           cut1_copy0 ? READWRITE : READONLY, 1);
+                           cut1_copy0 ? READWRITE : READONLY, 1, hopt);
 
-      /* If the output isn't opened yet, open it.  */
+      /* If the output isn't opened yet, open it. */
       if(out==NULL)
-        out = ( ( gal_fits_hdu_format(p->input->v, hdu)==IMAGE_HDU
+        out = ( ( gal_fits_hdu_format(p->input->v, hdu, hopt)==IMAGE_HDU
                   && p->primaryimghdu )
                 ? fits_open_to_write_no_blank(p->cp.output)
                 : gal_fits_open_to_write(p->cp.output) );
@@ -858,6 +884,9 @@ fits_hdu_copy(struct fitsparams *p, int cut1_copy0, int *r)
 
 
 
+/***************************************************************/
+/*************         Top-level function          *************/
+/***************************************************************/
 int
 fits(struct fitsparams *p)
 {
@@ -877,7 +906,6 @@ fits(struct fitsparams *p)
 
       /* Options that must be called alone. */
       if(p->numhdus)               fits_hdu_number(p);
-      else if(p->datasum)          fits_datasum(p);
       else if(p->pixelscale)       fits_pixelscale(p);
       else if(p->pixelareaarcsec2) fits_pixelarea(p);
       else if(p->skycoverage)      fits_skycoverage(p);
@@ -886,6 +914,7 @@ fits(struct fitsparams *p)
       else if(p->listimagehdus)    fits_certain_hdu(p, 1, 0);
       else if(p->listtablehdus)    fits_certain_hdu(p, 1, 1);
       else if(p->listallhdus)      fits_list_all_hdus(p);
+      else if(p->datasum || p->datasumencoded) fits_datasum(p);
 
       /* Options that can be called together. */
       else
@@ -913,9 +942,9 @@ fits(struct fitsparams *p)
 
     /* Not recognized. */
     default:
-      error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s to address "
-            "the problem. The code %d is not recognized for p->mode",
-            __func__, PACKAGE_BUGREPORT, p->mode);
+      error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s to "
+            "address the problem. The code %d is not recognized for "
+            "p->mode", __func__, PACKAGE_BUGREPORT, p->mode);
     }
 
   return r;
