@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2023 Free Software Foundation, Inc.
+Copyright (C) 2016-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -165,6 +165,8 @@ enum clumpcols
     CCOL_SIGCLIPMEDIAN,  /* Sigma-clipped mean of this clump.         */
     CCOL_RIV_NUM,        /* Num river pixels around this clump.       */
     CCOL_RIV_SUM,        /* Sum of rivers around clump.               */
+    CCOL_RIV_MIN,        /* Minimum of rivers around clump.           */
+    CCOL_RIV_MAX,        /* Maximum of rivers around clump.           */
     CCOL_RIV_SUM_VAR,    /* Variance of sum (for error measurements). */
     CCOL_VX,             /* Sum of (value-sky) * x.                   */
     CCOL_VY,             /* Sum of (value-sky) * y.                   */
@@ -260,12 +262,18 @@ struct mkcatalogparams
   uint8_t         noclumpsort;  /* Don't sort the clumps catalog.       */
   float             zeropoint;  /* Zero-point magnitude of object.      */
   uint8_t            variance;  /* Input STD file is actually variance. */
-  uint8_t        forcereadstd;  /* Read STD even if not needed.         */
+  uint8_t        novalinerror;  /* Ignore values when estimating errors.*/
   uint8_t         subtractsky;  /* ==1: subtract the Sky from values.   */
-  float           sfmagnsigma;  /* Surface brightness multiple of sigma.*/
-  float             sfmagarea;  /* Surface brightness area (arcsec^2).  */
   uint8_t       inbetweenints;  /* Keep rows (integer ids) with no labs.*/
   double         sigmaclip[2];  /* Sigma clip column settings.          */
+  uint8_t        metameasures;  /* Measurments on/about measurements.   */
+
+  float              sblsigma;  /* Surface brightness multiple of sigma.*/
+  float               sblarea;  /* Surface brightness area (arcsec^2).  */
+  float              nmlsigma;  /* Noise Mag. Lim. multiple of sigma.   */
+  float               nmlarea;  /* Noise Mag. Lim. area (arcsec^2).     */
+  uint8_t      cnlwithobjects;  /* Confusion limit should use objects.  */
+  uint8_t            cnlcheck;  /* Check table for positions.           */
 
   char            *upmaskfile;  /* Name of upper limit mask file.       */
   char             *upmaskhdu;  /* HDU of upper limit mask file.        */
@@ -279,7 +287,6 @@ struct mkcatalogparams
   float     spatialresolution;  /* Error in area (used in SB error).    */
 
   /* Internal. */
-  char           *relabclumps;  /* Name of new file for clump labels.   */
   time_t              rawtime;  /* Starting time of the program.        */
   gal_data_t          *values;  /* Input.                               */
   gal_data_t         *objects;  /* Object labels.                       */
@@ -289,16 +296,17 @@ struct mkcatalogparams
   gal_data_t          *upmask;  /* Upper limit magnitude mask.          */
   float                medstd;  /* Median standard deviation value.     */
   float               cpscorr;  /* Counts-per-second correction.        */
-  int32_t            *outlabs;  /* Labels in output cat (when necessary)*/
-  int32_t         *outlabsinv;  /* Inverse of the 'outlabs' array.      */
-  size_t           numobjects;  /* Number of object labels in image.    */
+  int32_t      *obj_from_tile;  /* Conv. of tile index to obj counter.  */
+  size_t         *obj_to_tile;  /* Conv. of obj counter to tile index.  */
+  int32_t         **origclpid;  /* Original clump labels in objects.    */
+  size_t          maxobjlabel;  /* Number of object labels in image.    */
+  size_t             numtiles;  /* Number of object labels in image.    */
   float               clumpsn;  /* Clump S/N threshold.                 */
   size_t            numclumps;  /* Number of clumps in image.           */
   gal_data_t      *objectcols;  /* Output columns for the objects.      */
   gal_data_t       *clumpcols;  /* Output columns for the clumps.       */
   gal_data_t           *tiles;  /* Tiles to cover each object.          */
-  char            *objectsout;  /* Output objects catalog.              */
-  char             *clumpsout;  /* Output clumps catalog.               */
+  gal_data_t         *upcheck;  /* Table contining the upper-limt check.*/
   char            *upcheckout;  /* Name of upperlimit check table.      */
   uint8_t             *oiflag;  /* Intermediate flags for objects.      */
   uint8_t             *ciflag;  /* Intermediate flags for clumps.       */
@@ -310,8 +318,8 @@ struct mkcatalogparams
   size_t               rngmin;  /* Minimum possible value of RNG.       */
   size_t              rngdiff;  /* Difference of RNG max and min.       */
   uint8_t      uprangewarning;  /* A warning must be printed.           */
-  size_t         *hostobjid_c;  /* To sort the clumps table by Obj.ID.  */
-  size_t         *numclumps_c;  /* To sort the clumps table by Obj.ID.  */
+  size_t          *hosttind_c;  /* A clump's host tile index.           */
+  size_t         *numclumps_c;  /* Number of clumps in a tile.          */
   double        pixelarcsecsq;  /* Area of input's pixels in arcsec^2.  */
 
   char        *usedvaluesfile;  /* Ptr to final name used for values.   */
