@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -35,7 +35,9 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/wcs.h>
 #include <gnuastro/data.h>
 #include <gnuastro/fits.h>
+#include <gnuastro/match.h>
 #include <gnuastro/units.h>
+#include <gnuastro/kdtree.h>
 #include <gnuastro/threads.h>
 #include <gnuastro/pointer.h>
 #include <gnuastro/dimension.h>
@@ -51,6 +53,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include "ui.h"
 #include "parse.h"
 #include "columns.h"
+#include "outkeys.h"
 #include "upperlimit.h"
 
 
@@ -190,6 +193,61 @@ mkcatalog_single_object_init(struct mkcatalogparams *p,
 
 
 
+/* Fill column values with blanks in case the label does not actually exist
+   in the input table (can happen with '--inbetweenints'). This is based on
+   the 'columns_fill' function. */
+static void
+mkcatalog_single_object_not_exist(struct mkcatalog_passparams *pp)
+{
+  void *vstart;
+  size_t i, oind;
+  gal_data_t *column;
+  struct mkcatalogparams *p=pp->p;
+
+  /* If the two metadata arrays are allocated, set them for this column. */
+  if(p->numclumps_c)
+    p->numclumps_c[ p->obj_to_tile
+                    ? p->obj_to_tile[pp->object]
+                    : pp->object-1 ] = 0;
+
+  /* Set this object's rows in all columns to blank. */
+  for(column=p->objectcols; column!=NULL; column=column->next)
+    {
+      /* This object's row index. */
+      oind=pp->object-1;
+
+      /* Write single-valued columns. */
+      switch(column->ndim)
+        {
+        case 1: /* Single value columns. */
+          gal_blank_write(gal_pointer_increment(column->array, oind,
+                                                column->type),
+                          column->type);
+          break;
+
+        case 2: /* Vector columns. */
+          vstart=gal_pointer_increment(column->array,
+                                      oind*column->dsize[1],
+                                      column->type);
+          for(i=0;i<column->dsize[1];++i)
+            gal_blank_write(gal_pointer_increment(vstart, i,
+                                                  column->type),
+                            column->type);
+          break;
+
+        default:
+          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
+                "'%s' to find and fix the problem. 'col->ndim' "
+                "should be either 1 or 2, but has a value of %zu",
+                __func__, PACKAGE_BUGREPORT, column->ndim);
+        }
+    }
+}
+
+
+
+
+
 /* Each thread will call this function once. It will go over all the
    objects that are assigned to it. */
 static void *
@@ -208,12 +266,19 @@ mkcatalog_single_object(void *in_prm)
   for(i=0; tprm->indexs[i]!=GAL_BLANK_SIZE_T; ++i)
     {
       /* For easy reading. Note that the object IDs start from one while
-         the array positions start from 0. */
-      pp.ci       = NULL;
-      pp.object   = ( p->outlabs
-                      ? p->outlabs[ tprm->indexs[i] ]
-                      : tprm->indexs[i] + 1 );
-      pp.tile     = &p->tiles[   tprm->indexs[i] ];
+         the tile indexs (that we parallelize over) start from 0. */
+      pp.ci = NULL;
+      pp.tile = &p->tiles[ tprm->indexs[i] ];
+      pp.object = ( p->obj_from_tile
+                    ? p->obj_from_tile[ tprm->indexs[i] ]
+                    : tprm->indexs[i] + 1 );
+
+      /* When the 'ndim' of the tile is zero, this label did not exist in
+         the image (only present because '--inbetweenints' was called). In
+         this case, simply set all the column values to blank and go to the
+         next object. */
+      if(pp.tile->ndim==0)
+        { mkcatalog_single_object_not_exist(&pp); continue; }
 
       /* Initialize the parameters for this object/tile. */
       parse_initialize(&pp);
@@ -398,303 +463,94 @@ mkcatalog_wcs_conversion(struct mkcatalogparams *p)
 
 
 
-void
-mkcatalog_outputs_keys_numeric(gal_fits_list_key_t **keylist, void *number,
-                               uint8_t type, char *nameliteral,
-                               char *commentliteral, char *unitliteral)
-{
-  void *value;
-  value=gal_pointer_allocate(type, 1, 0, __func__, "value");
-  memcpy(value, number, gal_type_sizeof(type));
-  gal_fits_key_list_add_end(keylist, type, nameliteral, 0,
-                            value, 1, commentliteral, 0,
-                            unitliteral, 0);
-}
-
-
-
-
-
-void
-mkcatalog_outputs_keys_infiles(struct mkcatalogparams *p,
-                               gal_fits_list_key_t **keylist)
-{
-  int quiet=p->cp.quiet;
-  char *stdname, *stdhdu, *stdvalcom;
-
-  gal_fits_key_list_title_add_end(keylist,
-                                  "Input files and/or configuration", 0);
-
-  /* Object labels. */
-  gal_fits_key_write_filename("INLAB", p->objectsfile, keylist, 0,
-                              quiet);
-  gal_fits_key_write_filename("INLABHDU", p->cp.hdu, keylist, 0,
-                              quiet);
-
-  /* Clump labels. */
-  if(p->clumps)
-    {
-      gal_fits_key_write_filename("INCLU", p->usedclumpsfile, keylist, 0,
-                                  quiet);
-      gal_fits_key_write_filename("INCLUHDU", p->clumpshdu, keylist, 0,
-                                  quiet);
-    }
-
-  /* Values image. */
-  if(p->values)
-    {
-      gal_fits_key_write_filename("INVAL", p->usedvaluesfile, keylist, 0,
-                                  quiet);
-      gal_fits_key_write_filename("INVALHDU", p->valueshdu, keylist, 0,
-                                  quiet);
-    }
-
-  /* Sky image/value. */
-  if(p->sky)
-    {
-      if(p->sky->size==1)
-        mkcatalog_outputs_keys_numeric(keylist, p->sky->array,
-                                       p->sky->type, "INSKYVAL",
-                                       "Value of Sky used (a single "
-                                       "number).", NULL);
-      else
-        {
-          gal_fits_key_write_filename("INSKY", p->usedskyfile, keylist, 0,
-                                      quiet);
-          gal_fits_key_write_filename("INSKYHDU", p->skyhdu, keylist, 0,
-                                      quiet);
-        }
-    }
-
-  /* Standard deviation (or variance) image. */
-  if(p->variance)
-    {
-      stdname="INVAR"; stdhdu="INVARHDU";
-      stdvalcom="Value of Sky variance (a single number).";
-    }
-  else
-    {
-      stdname="INSTD"; stdhdu="INSTDHDU";
-      stdvalcom="Value of Sky STD (a single number).";
-    }
-  if(p->std)
-    {
-      if(p->std->size==1)
-        mkcatalog_outputs_keys_numeric(keylist, p->std->array, p->std->type,
-                                       stdname, stdvalcom, NULL);
-      else
-        {
-          gal_fits_key_write_filename(stdname, p->usedstdfile, keylist, 0,
-                                      quiet);
-          gal_fits_key_write_filename(stdhdu, p->stdhdu, keylist, 0,
-                                      quiet);
-        }
-    }
-
-  /* Upper limit mask. */
-  if(p->upmaskfile)
-    {
-      gal_fits_key_write_filename("INUPM", p->upmaskfile, keylist, 0,
-                                  quiet);
-      gal_fits_key_write_filename("INUPMHDU", p->upmaskhdu, keylist, 0,
-                                  quiet);
-    }
-}
-
-
-
-
-
-/* Write the output keywords. */
-static gal_fits_list_key_t *
-mkcatalog_outputs_keys(struct mkcatalogparams *p, int o0c1)
-{
-  float pixarea=NAN, fvalue;
-  gal_fits_list_key_t *keylist=NULL;
-
-  /* First, add the file names. */
-  mkcatalog_outputs_keys_infiles(p, &keylist);
-
-  /* Type of catalog. */
-  gal_fits_key_list_add_end(&keylist, GAL_TYPE_STRING, "CATTYPE", 0,
-                            o0c1 ? "clumps" : "objects", 0,
-                            "Type of catalog ('object' or 'clump').", 0,
-                            NULL, 0);
-
-  /* Add project commit information when in a Git-controlled directory and
-     the output isn't a FITS file (in FITS, this will be automatically
-     included). */
-  if(p->cp.tableformat==GAL_TABLE_FORMAT_TXT && gal_git_describe())
-    gal_fits_key_list_add_end(&keylist, GAL_TYPE_STRING, "COMMIT", 0,
-                              gal_git_describe(), 1,
-                              "Git commit in running directory.", 0,
-                              NULL, 0);
-
-  /* Pixel area. */
-  if(p->objects->wcs)
-    {
-      pixarea=gal_wcs_pixel_area_arcsec2(p->objects->wcs);
-      if( isnan(pixarea)==0 )
-        mkcatalog_outputs_keys_numeric(&keylist, &pixarea,
-                                       GAL_TYPE_FLOAT32, "PIXAREA",
-                                       "Pixel area of input image.",
-                                       "arcsec^2");
-    }
-
-  /* Zeropoint magnitude. */
-  if( !isnan(p->zeropoint) )
-    mkcatalog_outputs_keys_numeric(&keylist, &p->zeropoint,
-                                   GAL_TYPE_FLOAT32, "ZEROPNT",
-                                   "Zeropoint used for magnitude.",
-                                   "mag");
-
-  /* Add the title for the keywords. */
-  gal_fits_key_list_title_add_end(&keylist, "Surface brightness limit "
-                                  "(SBL)", 0);
-
-  /* Print surface brightness limit. */
-  if( !isnan(p->medstd) && !isnan(p->sfmagnsigma) )
-    {
-      /* Used noise value (per pixel) and multiple of sigma. */
-      mkcatalog_outputs_keys_numeric(&keylist, &p->medstd,
-                                     GAL_TYPE_FLOAT32, "SBLSTD",
-                                     "Pixel STD for surface brightness "
-                                     "limit.", NULL);
-      mkcatalog_outputs_keys_numeric(&keylist, &p->sfmagnsigma,
-                                     GAL_TYPE_FLOAT32, "SBLNSIG",
-                                     "Sigma multiple for surface "
-                                     "brightness limit.", NULL);
-
-      /* Only print magnitudes if a zeropoint is given. */
-      if( !isnan(p->zeropoint) )
-        {
-          /* Per pixel, Surface brightness limit magnitude. */
-          fvalue=gal_units_counts_to_mag(p->sfmagnsigma * p->medstd,
-                                         p->zeropoint);
-          mkcatalog_outputs_keys_numeric(&keylist, &fvalue,
-                                         GAL_TYPE_FLOAT32, "SBLMAGPX",
-                                         "Surface brightness limit per "
-                                         "pixel.", "mag/pix");
-
-          /* Only print the SBL in fixed area if a WCS is present and a
-             pixel area could be deduced. */
-          if( !isnan(pixarea) )
-            {
-              /* Area used for measuring SBL. */
-              mkcatalog_outputs_keys_numeric(&keylist, &p->sfmagarea,
-                                             GAL_TYPE_FLOAT32, "SBLAREA",
-                                             "Area for surface brightness "
-                                             "limit.", "arcsec^2");
-
-              /* Per area, Surface brightness limit magnitude. */
-              fvalue=gal_units_counts_to_mag(p->sfmagnsigma
-                                             * p->medstd
-                                             / sqrt( p->sfmagarea
-                                                     * pixarea),
-                                             p->zeropoint);
-              mkcatalog_outputs_keys_numeric(&keylist, &fvalue,
-                                             GAL_TYPE_FLOAT32, "SBLMAG",
-                                             "Surf. bright. limit in "
-                                             "SBLAREA.",
-                                             "mag/arcsec^2");
-            }
-          else
-            gal_fits_key_list_fullcomment_add_end(&keylist, "Can't "
-                   "write surface brightness limiting magnitude (SBLM) "
-                   "values in fixed area ('SBLAREA' and 'SBLMAG' "
-                   "keywords) because input doesn't have a world "
-                   "coordinate system (WCS), or the first two "
-                   "coordinates of the WCS weren't angular positions "
-                   "in units of degrees.", 0);
-        }
-      else
-        gal_fits_key_list_fullcomment_add_end(&keylist, "Can't write "
-               "surface brightness limiting magnitude values (e.g., "
-               "'SBLMAG' or 'SBLMAGPX' keywords) because no "
-               "'--zeropoint' has been given.", 0);
-    }
-  else
-    {
-      gal_fits_key_list_fullcomment_add_end(&keylist, "No surface "
-             "brightness calcuations (e.g., 'SBLMAG' or 'SBLMAGPX' "
-             "keywords) because STD image didn't have the 'MEDSTD' "
-             "keyword. There are two solutions: 1) Call with "
-             "'--forcereadstd'. 2) Measure the median noise level "
-             "manually (possibly with Gnuastro's Arithmetic program) "
-             "and put the value in the 'MEDSTD' keyword of the STD "
-             "image.", 0);
-      gal_fits_key_list_fullcomment_add_end(&keylist, "", 0);
-    }
-
-  /* The count-per-second correction. */
-  if(p->cpscorr>1.0f)
-    mkcatalog_outputs_keys_numeric(&keylist, &p->cpscorr,
-                                   GAL_TYPE_FLOAT32, "CPSCORR",
-                                   "Counts-per-second correction.",
-                                   NULL);
-
-  /* Print upper-limit parameters. */
-  if(p->upperlimit)
-    upperlimit_write_keys(p, &keylist, 1);
-
-  /* In plain-text outputs, put a title for column metadata. */
-  if(p->cp.tableformat==GAL_TABLE_FORMAT_TXT)
-    gal_fits_key_list_title_add_end(&keylist, "Column metadata", 0);
-
-  /* Return the list of keywords. */
-  return keylist;
-}
-
-
-
-
-
 /* Since all the measurements were done in parallel (and we didn't know the
    number of clumps per object a-priori), the clumps informtion is just
    written in as they are measured. Here, we'll sort the clump columns by
-   object ID. There is an option to disable this. */
+   object ID (when it is requested). */
 static void
 sort_clumps_by_objid(struct mkcatalogparams *p)
 {
   gal_data_t *col;
-  size_t o, i, j, *permute, *rowstart;
+  size_t i, j, tind, *permute, *rowstart;
+  size_t ptwc=GAL_BLANK_SIZE_T; /* previous tile with clump. */
 
   /* Make sure everything is fine. */
-  if(p->hostobjid_c==NULL || p->numclumps_c==NULL)
-    error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
-          "problem. 'p->hostobjid_c' and 'p->numclumps_c' must not be "
-          "NULL.", __func__, PACKAGE_BUGREPORT);
-
+  if(p->hosttind_c==NULL || p->numclumps_c==NULL)
+    error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
+          "the problem. 'p->hostobjid_c' and 'p->numclumps_c' must not "
+          "be NULL.", __func__, PACKAGE_BUGREPORT);
 
   /* Allocate the necessary arrays. */
-  rowstart=gal_pointer_allocate(GAL_TYPE_SIZE_T, p->numobjects, 0, __func__,
-                                 "rowstart");
-  permute=gal_pointer_allocate(GAL_TYPE_SIZE_T, p->numclumps, 0, __func__,
-                                "permute");
+  rowstart=gal_pointer_allocate(GAL_TYPE_SIZE_T, p->numtiles, 0,
+                                __func__, "rowstart");
+  permute=gal_pointer_allocate(GAL_TYPE_SIZE_T, p->numclumps, 0,
+                               __func__, "permute");
 
+  /* The tile/object array is already sorted by tile ID. So we should just
+     add up the number of clumps in each object to find the row where each
+     object's clumps should start from in the final sorted clumps
+     catalog. */
+  for(i=0;i<p->numtiles;++i)
+    {
+      /* This tile has clumps, use the previous tile's total clumps to find
+         the starting row of this tile's clumps. */
+      if(p->numclumps_c[i])
+        {
+          rowstart[i] = ( ptwc==GAL_BLANK_SIZE_T
+                          ? 0
+                          : rowstart[ptwc]+p->numclumps_c[ptwc] );
+          ptwc=i;
 
-  /* The objects array is already sorted by object ID. So we should just
-     add up the number of clumps to find the row where each object's clumps
-     should start from in the final sorted clumps catalog. */
-  rowstart[0] = 0;
-  for(i=1;i<p->numobjects;++i)
-    rowstart[i] = p->numclumps_c[i-1] + rowstart[i-1];
+          /* For a check.
+          printf("%s: %zu, %zu, %zu\n", __func__, i,
+                 p->numclumps_c[i], rowstart[i]);
+          */
+        }
 
-  /* Fill the permutation array. Note that WE KNOW that all the objects for
-     one clump are after each other.*/
+      /* There are no clumps on this tile, so we'll just put a blank value
+         to create a segmentation fault if not used properly (better than a
+         logical error). */
+      else rowstart[i] = GAL_BLANK_SIZE_T;
+    }
+
+  /* Fill the permutation array to rearrange the clump rows. Note that WE
+     KNOW that all the clumps within for one objects are already
+     immediately after each other. */
   i=0;
   while(i<p->numclumps)
     {
-      o = ( p->outlabsinv
-            ? (p->outlabsinv[ p->hostobjid_c[i] ] + 1)
-            : p->hostobjid_c[i] ) - 1;
-      for(j=0; j<p->numclumps_c[o]; ++j)
-        permute[i++] = rowstart[o] + j;
+      /* Find the tile-id of this clump and depending on how many clumps
+         there are in this tile, increment the clump-index ('i'). */
+      tind = p->hosttind_c[i];
+      if(tind==GAL_BLANK_SIZE_T)
+        error(EXIT_FAILURE, 0, "the number of clumps in the image does "
+              "not match the 'NUMLABS' keyword of the input clumps HDU. "
+              "This can happen when the clumps have been derived from "
+              "Segment's output over one image while the object labels "
+              "were derived from another image. As a result, there are "
+              "multiple separate clumps with the same label but not with "
+              "different object labels. If this the case, the safest "
+              "solution is to re-run Segment with the '--noobjects' "
+              "option for generating the clump labels independent of any "
+              "objects (which will also be faster!). If you can no "
+              "longer run Segment, you can re-label the clumps with this "
+              "command: 'astarithmetic file-with-clumps.fits -hCLUMPS 0 "
+              "gt 2 connected-components --output=out.fits' and replace "
+              "the output with the clumps image you gave in this run");
+      for(j=0; j<p->numclumps_c[tind]; ++j)
+        {
+          permute[i++] = rowstart[tind] + j;
+
+          /* For a check.
+             printf("%s: clump-%zu --> obj-%zu: %zu\n", __func__, i-1,
+                    tind, permute[i-1]);
+          */
+        }
     }
 
   /* Permute all the clump columns. */
   for(col=p->clumpcols; col!=NULL; col=col->next)
-    gal_permutation_apply_inverse(col, permute);
+      gal_permutation_apply_inverse(col, permute);
 
   /* Clean up */
   free(permute);
@@ -709,63 +565,61 @@ sort_clumps_by_objid(struct mkcatalogparams *p)
 static void
 mkcatalog_write_outputs(struct mkcatalogparams *p)
 {
-  gal_fits_list_key_t *keylist;
+  char *out=p->cp.output;
+  int tf=p->cp.tableformat;
+  gal_fits_list_key_t *tmp;
   gal_list_str_t *comments=NULL;
-  int outisfits=gal_fits_name_is_fits(p->objectsout);
+  char *ohdu="OBJECTS", *chdu="CLUMPS", *cuhdu="CHECK-UPPERLIMIT";
 
   /* If a catalog is to be generated. */
   if(p->objectcols)
     {
-      /* OBJECT catalog */
-      keylist=mkcatalog_outputs_keys(p, 0);
-
       /* Reverse the comments list (so it is printed in the same order
          here), write the objects catalog and free the comments. */
       gal_list_str_reverse(&comments);
-      gal_table_write(p->objectcols, &keylist, NULL, p->cp.tableformat,
-                      p->objectsout, "OBJECTS", 0);
+      gal_table_write(p->objectcols, NULL, NULL, tf, out, ohdu, 0, 1);
       gal_list_str_free(comments, 1);
 
-
       /* CLUMPS catalog */
       if(p->clumps)
         {
-          /* Make the comments. */
-          keylist=mkcatalog_outputs_keys(p, 1);
-
-          /* Write objects catalog
-             ---------------------
-
-             Reverse the comments list (so it is printed in the same order
+          /* Reverse the comments list (so it is printed in the same order
              here), write the objects catalog and free the comments. */
           gal_list_str_reverse(&comments);
-          gal_table_write(p->clumpcols, NULL, comments, p->cp.tableformat,
-                          p->clumpsout, "CLUMPS", 0);
+          gal_table_write(p->clumpcols, NULL, comments, tf, out, chdu,
+                          0, 1);
           gal_list_str_free(comments, 1);
         }
+
+      /* Upper limit check table. */
+      if(p->upcheck)
+        gal_table_write(p->upcheck, NULL, comments, tf, out, cuhdu, 0, 1);
     }
 
   /* Configuration information. */
-  if(outisfits)
-    {
-      gal_fits_key_write_filename("input", p->objectsfile, &p->cp.okeys,
-                                  1, p->cp.quiet);
-      gal_fits_key_write_config(&p->cp.okeys, "MakeCatalog configuration",
-                                "MKCATALOG-CONFIG", p->objectsout, "0");
-    }
+  gal_fits_key_write_filename("input", p->objectsfile, &p->cp.ckeys,
+                              1, p->cp.quiet);
 
+  /* MakeCatalog's analysis/measurement headers after the configuration
+     (input options) headers. */
+  for(tmp=p->cp.ckeys; tmp!=NULL; tmp=tmp->next)
+    if(tmp->next==NULL) { tmp->next=outkeys_write(p); break; }
+
+  /* Write all the keywords in the 0-th HDU of the output. This function
+     also frees the whole key list. */
+  gal_fits_key_write(p->cp.ckeys, out, "0", "NONE", 1, 0);
 
   /* Inform the user */
-  if(!p->cp.quiet && p->objectcols)
+  if(!p->cp.quiet)
     {
-      if(p->clumpsout && strcmp(p->clumpsout,p->objectsout))
-        {
-          printf("  - Output objects catalog: %s\n", p->objectsout);
-          if(p->clumps)
-            printf("  - Output clumps catalog: %s\n", p->clumpsout);
-        }
-      else
-        printf("  - Catalog written to %s\n", p->objectsout);
+      printf("  - Output written to (HDUs listed below): %s\n"
+             "    - OBJECTS: measurements on the object labels.\n",
+             out);
+      if(p->clumps)
+        printf("    - CLUMPS: measurements on the clump labels.\n");
+      if(p->upcheck)
+        printf("    - CHECK-UPPERLIMIT: measurements on the "
+               "clump labels.\n");
     }
 }
 
@@ -799,7 +653,7 @@ mkcatalog(struct mkcatalogparams *p)
   if( p->cp.numthreads > 1 ) pthread_mutex_init(&p->mutex, NULL);
 
   /* Do the processing on each thread. */
-  gal_threads_spin_off(mkcatalog_single_object, p, p->numobjects,
+  gal_threads_spin_off(mkcatalog_single_object, p, p->numtiles,
                        p->cp.numthreads, p->cp.minmapsize,
                        p->cp.quietmmap);
 
@@ -808,9 +662,8 @@ mkcatalog(struct mkcatalogparams *p)
   mkcatalog_wcs_conversion(p);
 
   /* If the columns need to be sorted (by object ID), then some adjustments
-     need to be made (possibly to both the objects and clumps catalogs). */
-  if(p->hostobjid_c)
-    sort_clumps_by_objid(p);
+     need to be made to the clumps catalog.. */
+  if(p->hosttind_c) sort_clumps_by_objid(p);
 
   /* Write the filled columns into the output. */
   mkcatalog_write_outputs(p);
