@@ -5,7 +5,7 @@ Segment is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -66,8 +66,8 @@ clumps_grow_prepare_initial(struct clumps_thread_params *cltprm)
   gal_data_t *input=cltprm->clprm->p->input;
   struct segmentparams *p=cltprm->clprm->p;
 
-  size_t ndiffuse=0, coord[2], *dindexs;
-  double wcoord[2]={0.0f,0.0f}, sum=0.0f;
+  double wcoord[3]={0.0f,0.0f,0.0f}, sum=0.0f;
+  size_t ndiffuse=0, coord[3], tcoord[3], *dindexs;
   size_t *s, *sf, *dsize=input->dsize, ndim=input->ndim;
   float glimit, *imgss=input->array, *std=p->std->array;
   int32_t *olabel=p->olabel->array, *clabel=p->clabel->array;
@@ -79,9 +79,12 @@ clumps_grow_prepare_initial(struct clumps_thread_params *cltprm)
   do
     if( imgss[ *s ] > 0.0f )
       {
-        sum        += imgss[ *s ];
-        wcoord[0]  += imgss[ *s ] * (*s/dsize[1]);
-        wcoord[1]  += imgss[ *s ] * (*s%dsize[1]);
+        gal_dimension_index_to_coord(*s, ndim, dsize, tcoord);
+        sum         += imgss[ *s ];
+        wcoord[0]   += imgss[ *s ] * tcoord[0];
+        wcoord[1]   += imgss[ *s ] * tcoord[1];
+        if(ndim==3)
+          wcoord[1] += imgss[ *s ] * tcoord[2];
       }
   while(++s<sf);
 
@@ -93,8 +96,11 @@ clumps_grow_prepare_initial(struct clumps_thread_params *cltprm)
       sf=(s=indexs->array)+indexs->size;
       do
         {
-          wcoord[0] += *s / dsize[1];
-          wcoord[1] += *s % dsize[1];
+          gal_dimension_index_to_coord(*s, ndim, dsize, tcoord);
+          wcoord[0]   += tcoord[0];
+          wcoord[1]   += tcoord[1];
+          if(ndim==3)
+            wcoord[2] += tcoord[2];
         }
       while(++s<sf);
       sum = indexs->size;
@@ -104,6 +110,8 @@ clumps_grow_prepare_initial(struct clumps_thread_params *cltprm)
   /* Convert floating point coordinates to integers. */
   coord[0] = GAL_DIMENSION_FLT_TO_INT(wcoord[0]/sum);
   coord[1] = GAL_DIMENSION_FLT_TO_INT(wcoord[1]/sum);
+  if(ndim==3)
+    coord[2] = GAL_DIMENSION_FLT_TO_INT(wcoord[2]/sum);
 
 
   /* Find the growth limit. Note that the STD may be a value, or a dataset
@@ -295,14 +303,15 @@ clumps_find_make_sn_table(void *in_prm)
 
   void *tarray;
   double numdet;
+  int pixonedge;
   gal_data_t *tile, *tblock, *tmp;
   uint8_t *binary=p->binary->array;
   struct clumps_thread_params cltprm;
-  size_t i, c, ind, tind, num, numsky, *indarr;
+  size_t i, j, c, ind, tind, num, numsky, *indarr;
   size_t *scoord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
-                                       "scoord");
+                                      "scoord");
   size_t *icoord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
-                                       "icoord");
+                                      "icoord");
 
 
   /* Initialize the parameters for this thread. */
@@ -392,12 +401,19 @@ clumps_find_make_sn_table(void *in_prm)
               ind = (int32_t *)i - (int32_t *)(p->clabel->array);
               gal_dimension_index_to_coord(ind, ndim, dsize, icoord);
 
-              /* If the pixel is on the tile edge, set it as river and
-                 don't include it in the indexs. */
-              if( icoord[0]==scoord[0]
-                  || icoord[0]==scoord[0]+tile->dsize[0]-1
-                  || icoord[1]==scoord[1]
-                  || icoord[1]==scoord[1]+tile->dsize[1]-1 )
+              /* Check if the pixel is on the tile edge. */
+              pixonedge=0;
+              for(j=0;j<ndim;++j)
+                if( icoord[j]==scoord[j]
+                    || icoord[j]==scoord[j]+tile->dsize[j]-1 )
+                  {
+                    pixonedge=1;
+                    break;
+                  }
+
+              /* If this pixel is on the edge, then it should be a
+                 river. */
+              if(pixonedge)
                 *(int32_t *)i=GAL_LABEL_RIVER;
 
               /* This pixel is not on the edge, check if it had a value of
@@ -515,9 +531,9 @@ clumps_write_sn_table(struct segmentparams *p, gal_data_t *insn,
     {
       /* Remove blank elements. */
       ind=gal_data_copy(inind);
+      gal_blank_remove(ind,1);
       sn=gal_data_copy(insn);
-      gal_blank_remove(ind);
-      gal_blank_remove(sn);
+      gal_blank_remove(sn,1);
 
       /* A small sanity check. */
       if(ind->size==0 || sn->size==0)
@@ -540,7 +556,7 @@ clumps_write_sn_table(struct segmentparams *p, gal_data_t *insn,
 
   /* write the table. */
   gal_table_write(cols, NULL, comments, p->cp.tableformat, filename,
-                  "SKY_CLUMP_SN", 0);
+                  "SKY_CLUMP_SN", 0, 0);
 
   /* Clean up (if necessary). */
   if(sn!=insn) gal_data_free(sn);
@@ -635,17 +651,17 @@ clumps_true_find_sn_thresh(struct segmentparams *p)
             case 1: p->clabel->name = "SKY_CLUMPS_ALL";    break;
             case 2: p->clabel->name = "SKY_CLUMPS_FOR_SN"; break;
             default:
-              error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s so "
-                    "we can address the issue. The value %d is not valid for "
-                    "clprm.step", __func__, PACKAGE_BUGREPORT, clprm.step);
+              error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s "
+                    "so we can address the issue. The value %d is not "
+                    "valid for clprm.step", __func__, PACKAGE_BUGREPORT,
+                    clprm.step);
             }
 
           /* Write the demonstration array into the check image. The
              default values are hard to view, so we'll make a copy of the
              demo, set all Sky regions to blank and all clump macro values
              to zero. */
-          gal_fits_img_write(p->clabel, p->segmentationname, NULL,
-                             PROGRAM_NAME);
+          gal_fits_img_write(p->clabel, p->segmentationname, NULL, 0);
 
           /* Increment the step counter. */
           ++clprm.step;
@@ -836,11 +852,7 @@ clumps_det_keep_true_relabel(struct clumps_thread_params *cltprm)
                  touching a river or not. */
               istouching=0;
               GAL_DIMENSION_NEIGHBOR_OP(cltprm->topinds[i], ndim, dsize,
-                                        ndim, dinc,
-                                        {
-                                          if(clabel[nind]==0)
-                                            istouching=1;
-                                        });
+                        ndim, dinc, { if(clabel[nind]<1) istouching=1; });
 
               /* If the peak isn't touching a river, then check its S/N and
                  if that is also good, give it a new label. */
