@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -93,6 +93,9 @@ gal_statistics_number(gal_data_t *input);
 gal_data_t *
 gal_statistics_minimum(gal_data_t *input);
 
+gal_data_t *
+gal_statistics_minimums_three(gal_data_t *input);
+
 gal_data_t *
 gal_statistics_maximum(gal_data_t *input);
 
@@ -199,6 +202,16 @@ gal_statistics_cfp(gal_data_t *data, gal_data_t *bins, int normalize);
 
 
 
+/****************************************************************
+ *****************     Distribution shape    ********************
+ ****************************************************************/
+gal_data_t *
+gal_statistics_concentration(gal_data_t *input, double width,
+                             int inplace);
+
+
+
+
 /****************************************************************
  *****************         Outliers          ********************
  ****************************************************************/
@@ -220,8 +233,8 @@ gal_statistics_outlier_bydistance(int pos1_neg0, gal_data_t *input,
 gal_data_t *
 gal_statistics_outlier_flat_cfp(gal_data_t *input, size_t numprev,
                                 float sigclip_multip, float sigclip_param,
-                                float thresh, size_t numcontig, int inplace,
-                                int quiet, size_t *index);
+                                float thresh, size_t numcontig,
+                                int inplace, int quiet, size_t *index);
 
 
 
