@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -184,7 +184,7 @@ gal_wcs_dimension_name(struct wcsprm *wcs, size_t dimension);
 void
 gal_wcs_write(struct wcsprm *wcs, char *filename,
               char *extname, gal_fits_list_key_t *headers,
-              char *program_string);
+              char *program_string, int freekeys);
 
 char *
 gal_wcs_write_wcsstr(struct wcsprm *wcs, int *nkeyrec);
