@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2023 Free Software Foundation, Inc.
+Copyright (C) 2016-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -31,6 +31,8 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <inttypes.h>
 
+#include <gsl/gsl_complex.h>
+
 #include <gnuastro/type.h>
 #include <gnuastro/data.h>
 #include <gnuastro/list.h>
@@ -70,26 +72,26 @@ gal_type_sizeof(uint8_t type)
 
     case GAL_TYPE_FLOAT32:
       if( sizeof (float) != 4 )
-        error(EXIT_FAILURE, 0, "%s: 'float' is not 32 bits on this machine",
-              __func__);
+        error(EXIT_FAILURE, 0, "%s: 'float' is not 32 bits on this "
+              "machine", __func__);
       return sizeof (float);
 
     case GAL_TYPE_FLOAT64:
       if( sizeof (double) != 8 )
-        error(EXIT_FAILURE, 0, "%s: 'double' is not 64 bits on this machine",
-              __func__);
+        error(EXIT_FAILURE, 0, "%s: 'double' is not 64 bits on this "
+              "machine", __func__);
       return sizeof (double);
 
     case GAL_TYPE_COMPLEX32:
       if( sizeof (float) != 4 )
-        error(EXIT_FAILURE, 0, "%s: 'float' is not 32 bits on this machine",
-              __func__);
+        error(EXIT_FAILURE, 0, "%s: 'float' is not 32 bits on this "
+              "machine", __func__);
       return sizeof (gsl_complex_float);
 
     case GAL_TYPE_COMPLEX64:
       if( sizeof (double) != 8 )
-        error(EXIT_FAILURE, 0, "%s: 'double' is not 64 bits on this machine",
-              __func__);
+        error(EXIT_FAILURE, 0, "%s: 'double' is not 64 bits on this "
+              "machine", __func__);
       return sizeof (gsl_complex);
 
     case GAL_TYPE_STRING:
@@ -100,9 +102,9 @@ gal_type_sizeof(uint8_t type)
             __func__, type);
     }
 
-  error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s so we can find "
-        "the cause of the problem. Control should not have reached the end of "
-        "this function", __func__, PACKAGE_BUGREPORT);
+  error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s so we can "
+        "find the cause of the problem. Control should not have reached "
+        "the end of this function", __func__, PACKAGE_BUGREPORT);
   return -1;
 }
 
