@@ -4,8 +4,9 @@
 # (in the Installing gnuastro section).
 #
 # Original author:
-#     Mohammad Akhlaghi <akhlaghi@gnu.org>
+#     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
+# Copyright (C) 2015-2025 Free Software Foundation, Inc.
 #
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
@@ -25,7 +26,7 @@
 prog=mkcatalog
 objimg=clearcanvas.fits
 execname=../bin/$prog/ast$prog
-img=convolve_spatial_noised_labeled.fits
+img=convolve_spatial_noised_detected.fits
 
 
 
@@ -51,6 +52,10 @@ if [ ! -f $objimg   ]; then echo "$objimg does not exist";  exit 77; fi
 
 # Actual test script
 # ==================
-$execname $img --objectsfile=$objimg --objectshdu=1 \
-          --output=aperturephot.txt                 \
-          --objid --x --y --ra --dec --magnitude --sn
+#
+# 'check_with_program' can be something like Valgrind or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
+$check_with_program $execname $objimg --hdu=1 --valuesfile=$img \
+                              --output=aperturephot.fits \
+                              --obj-id --x --y --ra --dec --magnitude --sn
